package com.qnapcomm.myapplication;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import com.qnap.qdk.qtshttp.IQtsHttpTransferedProgressListener;
import com.qnap.qdk.qtshttp.QtsHttpCancelController;
import com.qnap.qdk.qtshttp.filestation.QtsHttpFileEntry;

import java.io.File;
import java.util.ArrayList;

import static android.os.Environment.DIRECTORY_DOWNLOADS;

public class EntryList extends Activity {
    private static final int sMsgUpdateListView = 1;
    private static final int sMsgUpdateProgress = 2;

    private Button btnUpload;
    private Button btnLogout;
    private TextView tvPath;
    private ListView lvEntry;

    private int mSelfRequestCode;
    private int mParentRequestCode;
    private String mParentPath;
    private String mUploadFile;
    private long mUploadSize;
    private ArrayList<QtsHttpFileEntry> mFileEntryList;

    public ProgressDialog mProgDlg = null;
    public TransferedProgressListener mProgListener = null;

    private QtsHttpFileEntry mFileEntry;


    // Upload/download progress listener
    public class TransferedProgressListener implements IQtsHttpTransferedProgressListener {
        private long mTotalFileLengthInBytes;

        public TransferedProgressListener() {
            mTotalFileLengthInBytes = 0;
        }

        @Override
        public void onProgress(long transferedFileLengthInBytes) {
            if (mTotalFileLengthInBytes > 0) {
                long percentage = Math.round(((double)(transferedFileLengthInBytes * 100) / mTotalFileLengthInBytes));
                Message msg = new Message();
                Bundle data = msg.getData();
                data.putLong("percent",  percentage);
                msg.what = sMsgUpdateProgress;
                msg.setData(data);
                uiMessageHandler.sendMessage(msg);
            }
        }

        public void setTotalFileLengthInBytes(long totalFileLengthInBytes) {
            this.mTotalFileLengthInBytes = totalFileLengthInBytes;
        }

    }

    public Handler uiMessageHandler = new Handler() {

        @Override
        public void handleMessage (Message msg) {
            switch (msg.what) {
                case sMsgUpdateListView:
                    mProgDlg.dismiss();
                    ArrayList<String> alFileEntryName = msg.getData().getStringArrayList("fileEntryName");
                    lvEntry.setAdapter(new ArrayAdapter<String>(EntryList.this, android.R.layout.simple_list_item_1, alFileEntryName));
                    lvEntry.invalidate();
                    break;
                case sMsgUpdateProgress:
                    int p = (int)msg.getData().getLong("percent");
                    mProgDlg.setProgress(p);
                    break;
            }

            super.handleMessage(msg);
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.entry_list);

        mSelfRequestCode = 0;

        Intent intent;
        Bundle bundle;

        intent = getIntent();
        bundle = intent.getExtras();
        mParentPath = bundle.getString("relativePath");
        mParentRequestCode = bundle.getInt("requestCode");
        mSelfRequestCode = mParentRequestCode + 1;

        CharSequence strDlgBodyLoading = getString(R.string.progdlg_body_loading);
        CharSequence strDlgTitle = getString(R.string.progdlg_title);

        mProgDlg = new ProgressDialog(EntryList.this);
        mProgDlg.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        mProgDlg.setTitle(strDlgTitle);
        mProgDlg.setMessage(strDlgBodyLoading);
        mProgDlg.show();

        Thread thread = new Thread() {
            @Override
            public void run() {
                mFileEntryList = null;
                QtsHttpCancelController cancel = new QtsHttpCancelController();
                try {
                    if (mParentPath.equals("/")) {
                        mFileEntryList = MainActivity.sFileStation.getShareFolderList(cancel);
                    } else {
                        mFileEntryList = MainActivity.sFileStation.getFileList(mParentPath, 0, 50, cancel);
                    }
                }
                catch (Exception e) {
                    e.printStackTrace();
                }
                finally {
                    ArrayList<String> alFileEntryName = new ArrayList<String>();
                    for (QtsHttpFileEntry fileEntry : mFileEntryList) {
                        String pathDisp;
                        if (fileEntry.isDir()) {
                            pathDisp = fileEntry.getFileName() + "/";
                        } else {
                            pathDisp = fileEntry.getFileName();
                        }
                        alFileEntryName.add(pathDisp);
                    }
                    Message msg = new Message();
                    Bundle data = msg.getData();
                    data.putStringArrayList("fileEntryName", alFileEntryName);
                    msg.what = sMsgUpdateListView;
                    msg.setData(data);
                    uiMessageHandler.sendMessage(msg);
                }
                super.run();
            }
        };
        thread.start();

        tvPath = (TextView)findViewById(R.id.tvPath);
        tvPath.setText("Path: " + mParentPath);

        lvEntry = (ListView) findViewById(R.id.lvEntry);
        lvEntry.setChoiceMode(ListView.CHOICE_MODE_NONE);
        lvEntry.setOnItemClickListener(new AdapterView.OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> a, View v, int pos, long id) {
                QtsHttpFileEntry fileEntry = mFileEntryList.get(pos);

                if (fileEntry.isDir()) {
                    Intent intent = new Intent();
                    intent.setClass(EntryList.this, EntryList.class);

                    String nextPath;
                    if (mParentPath.equals("/")) {
                        nextPath = "/" + fileEntry.getFileName();
                    } else {
                        nextPath = mParentPath + "/" + fileEntry.getFileName();
                    }

                    Bundle bundle = new Bundle();
                    bundle.putString("relativePath", nextPath);
                    bundle.putInt("requestCode", mSelfRequestCode);

                    intent.putExtras(bundle);

                    startActivity(intent);
                }
            }
        });
        lvEntry.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
            public boolean onItemLongClick(AdapterView<?> a, View v, int pos, long id)
            {
                QtsHttpFileEntry fileEntry = mFileEntryList.get(pos);
                mFileEntry = fileEntry;

                if (!mParentPath.equals("/"))
                {
                    if (!fileEntry.isDir())
                    {
                        CharSequence strDlgTitle = getString(R.string.progdlg_title);
                        CharSequence strDlgBody = getString(R.string.progdlg_body_download);

                        mProgDlg = new ProgressDialog(EntryList.this);
                        mProgDlg.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
                        mProgDlg.setMax(100);
                        mProgDlg.setProgress(0);
                        mProgDlg.setMessage(strDlgBody);
                        mProgDlg.setTitle(strDlgTitle);
                        mProgDlg.setCancelable(false);
                        mProgDlg.show();

                        mProgListener = new TransferedProgressListener();

                        new Thread()
                        {
                            public void run()
                            {
                                QtsHttpCancelController cancel = new QtsHttpCancelController();
                                try
                                {
                                    File downloadFolder = Environment.getExternalStoragePublicDirectory(DIRECTORY_DOWNLOADS);
                                    String downloadDestPath = downloadFolder.getPath() +File.separator +  mFileEntry.getFileName();
                                    String downloadSrcPath = mFileEntry.getFilePath() + File.separator + mFileEntry.getFileName();
                                    mProgListener.setTotalFileLengthInBytes(mFileEntry.getFileSize());
                                    MainActivity.sFileStation.downloadFileByPath(downloadSrcPath, downloadDestPath, 0, cancel, mProgListener);
                                }
                                catch (Exception e)
                                {
                                    e.printStackTrace();
                                }
                                finally
                                {
                                    mProgDlg.dismiss();
                                    Intent intent = new Intent();
                                    intent.setClass(EntryList.this, EntryList.class);

                                    Bundle bundle = new Bundle();
                                    bundle.putString("relativePath", mParentPath);
                                    bundle.putInt("requestCode", mParentRequestCode);

                                    intent.putExtras(bundle);

                                    startActivity(intent);
                                    finish();
                                }
                            }
                        }.start();
                    }
                }

                return true;
            }
        });

        btnUpload = (Button)findViewById(R.id.btnUpload);
        btnUpload.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent();
                intent.setClass(EntryList.this, LocalList.class);

                Bundle bundle = new Bundle();
                bundle.putInt("requestCode", mSelfRequestCode);

                intent.putExtras(bundle);

                startActivityForResult(intent, mSelfRequestCode);
            }
        });

        btnLogout = (Button)findViewById(R.id.btnLogout);
        btnLogout.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent();
                intent.setClass(EntryList.this, MainActivity.class);

                Bundle bundle = new Bundle();
                bundle.putString("func", "logout");

                intent.putExtras(bundle);

                startActivity(intent);
                finish();
            }
        });
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode == mSelfRequestCode)
        {
            mUploadFile = data.getExtras().getString("uploadFile");
            if (!mUploadFile.equals("")) {
                mUploadSize = data.getExtras().getLong("uploadSize");

                CharSequence strDlgTitle = getString(R.string.progdlg_title);
                CharSequence strDlgBody = getString(R.string.progdlg_body_upload);

                mProgDlg = new ProgressDialog(EntryList.this);
                mProgDlg.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
                mProgDlg.setMax(100);
                mProgDlg.setProgress(0);
                mProgDlg.setMessage(strDlgBody);
                mProgDlg.setTitle(strDlgTitle);
                mProgDlg.setCancelable(false);
                mProgDlg.show();

                mProgListener = new TransferedProgressListener();
                mProgListener.setTotalFileLengthInBytes(mUploadSize);

                new Thread()
                {
                    public void run()
                    {
                        try
                        {
                            QtsHttpCancelController cancel = new QtsHttpCancelController();
                            MainActivity.sFileStation.uploadFileByPath(mUploadFile, mParentPath, cancel, mProgListener);
                        }
                        catch (Exception e)
                        {
                            e.printStackTrace();
                        }
                        finally
                        {
                            try {
                                mProgDlg.dismiss();
                                Intent intent = new Intent();
                                intent.setClass(EntryList.this, EntryList.class);

                                Bundle bundle = new Bundle();
                                bundle.putString("relativePath", mParentPath);
                                bundle.putInt("requestCode", mParentRequestCode);

                                intent.putExtras(bundle);

                                startActivity(intent);
                                finish();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }.start();
            }
        }
    }
}
