package com.qnapcomm.myapplication;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Toast;

import com.qnap.qdk.qtshttp.QtsHttpServer;
import com.qnap.qdk.qtshttp.QtsHttpServerInfo;
import com.qnap.qdk.qtshttp.QtsHttpStationType;
import com.qnap.qdk.qtshttp.exception.QtsHttpAuthorizationFailedException;
import com.qnap.qdk.qtshttp.exception.QtsHttpNetworkTimeoutException;
import com.qnap.qdk.qtshttp.filestation.IQtsHttpFileStation;

import androidx.appcompat.app.AppCompatActivity;

public class MainActivity extends AppCompatActivity {
    private static final int sMsgLogining = 0;
    private static final int sMsgLogin = 1;
    private static final int sMsgLogout = 2;
    private static final int sMsgCloseApp = 3;

    private EditText etServerIP;
    private EditText etPort;
    private EditText etUserName;
    private EditText etPassword;
    private CheckBox cbxIsSSL;

    public static IQtsHttpFileStation sFileStation;

    public ProgressDialog mProgDlg = null;
    private QtsHttpServer mServer;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Intent intent;
        Bundle bundle;
        String func = "";

        // Check logout
        intent = getIntent();
        if (intent != null) {
            bundle = intent.getExtras();
            if (bundle != null) {
                func = bundle.getString("func");
                if (!TextUtils.isEmpty(func) && func.equals("logout")) {
                    Thread thread = new Thread() {
                        @Override
                        public void run() {
                            try {
                                mServer.closeFileStation(sFileStation);
                                sFileStation = null;
                                mServer = null;
                            } catch (Exception e) {
                                e.printStackTrace();
                            }

                            // Send message to UI to show logout success message
                            Message msg = new Message();
                            msg.what = sMsgLogout;
                            uiMessageHandler.sendMessage(msg);
                        }
                    };
                    thread.start();
                }
            }
        }


        ((Button)findViewById(R.id.btnLogin)).setOnClickListener(login);
        etServerIP = (EditText)findViewById(R.id.etServerIP);         // Server IP
        etPort = (EditText)findViewById(R.id.etPort);                 // Port
        etUserName = (EditText)findViewById(R.id.etUserName);         // User name
        etPassword = (EditText)findViewById(R.id.etPassword);         // Password
        cbxIsSSL = (CheckBox)findViewById(R.id.cbxIsSSL);             // Is SSL

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    public Handler uiMessageHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            String message = "";
            Toast toast;
            switch (msg.what) {
                // Logining
                case sMsgLogining:
                    CharSequence strDlgBodyLoading = getString(R.string.progdlg_body_logining);
                    CharSequence strDlgTitle = getString(R.string.progdlg_title);

                    mProgDlg = new ProgressDialog(MainActivity.this);
                    mProgDlg.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                    mProgDlg.setTitle(strDlgTitle);
                    mProgDlg.setMessage(strDlgBodyLoading);
                    mProgDlg.show();
                    break;
                // Login result
                case sMsgLogin:
                    mProgDlg.dismiss();
                    boolean isLogin = msg.getData().getBoolean("isLogin");
                    message = msg.getData().getString("message");
                    toast = Toast.makeText(MainActivity.this, message, Toast.LENGTH_SHORT);
                    toast.show();
                    break;
                // Logout
                case sMsgLogout:
                    message ="Logout";
                    toast = Toast.makeText(MainActivity.this, message, Toast.LENGTH_SHORT);
                    toast.show();
                    break;
                // Close application
                case sMsgCloseApp:
                    break;
            }
        }
    };

    // Login
    private View.OnClickListener login = new View.OnClickListener() {

        @Override
        public void onClick(View v) {
            Thread thread = new Thread() {
                @Override
                public void run() {
                    String serverIP = etServerIP.getText().toString();        // Server IP
                    String userName = etUserName.getText().toString();        // User name
                    String password = etPassword.getText().toString();        // Password
                    String agentName = "QNAP QtsHttp Java Sample Code for Android 1.0.0";  // Agent name
                    boolean isSSL = cbxIsSSL.isChecked();                     // Use SSL
                    int port = Integer.valueOf(etPort.getText().toString());  // Port
                    QtsHttpServerInfo serverInfo = new QtsHttpServerInfo(serverIP, userName, password, isSSL, "");

                    mServer = new QtsHttpServer(serverInfo);
                    try {
                        if (isSSL) {
                            mServer.setFileStationSSLPortNum(port);
                        } else {
                            mServer.setFileStationPortNum(port);
                        }
                        mServer.setAgentName(agentName);
                    } catch (Exception e1) {
                        e1.printStackTrace();
                    }

                    Message msg = new Message();
                    Bundle data = msg.getData();
                    boolean isLogin = false;
                    try {
                        // Send message to UI thread to show Loging dialog
                        Message msgLogining = new Message();
                        msgLogining.what = sMsgLogining;
                        uiMessageHandler.sendMessage(msgLogining);
                        // Login to File Station on NAS
                        mServer.login(QtsHttpStationType.QTS_HTTP_STATION_TYPE_FILE_STATION);

                        // Login success
                        data.putString("message", "Login success.");
                        data.putBoolean("isLogin", true);
                        sFileStation = mServer.openFileStation();
                        isLogin = true;
                    } catch (QtsHttpNetworkTimeoutException timeoute) {
                        // Network timeout
                        data.putString("message", "Timeout.");
                        data.putBoolean("isLogin", false);
                    } catch (QtsHttpAuthorizationFailedException authe) {
                        // Login fail
                        data.putString("message", "Login fail.");
                        data.putBoolean("isLogin", false);
                    } catch (Exception e) {
                        e.printStackTrace();
                        data.putString("message", "Error.");
                        data.putBoolean("isLogin", false);
                        e.printStackTrace();
                    }
                    finally {

                    }
                    // Send message to UI thread to show login result
                    msg.what = sMsgLogin;
                    msg.setData(data);
                    uiMessageHandler.sendMessage(msg);

                    // If login to NAS successfully, goto show NAS share folders
                    if (isLogin) {
                        Intent intent = new Intent();
                        intent.setClass(MainActivity.this, EntryList.class);

                        Bundle bundle = new Bundle();
                        bundle.putString("relativePath", "/");

                        intent.putExtras(bundle);

                        startActivity(intent);
                    }
                }
            };
            thread.start();
        }
    };

}